#Author Davood Mazinanian

printCasesWithTimeMoreThan <- function(seconds) {
  test<-data.frame(id=joined$ID.x[joined$totalTime / 1e9 > seconds], tool=joined$projectName[joined$totalTime / 1e9 > seconds]) 
  print(test[with(test, order(tool, id)), ])
  rm(test)
}

printTimesInClusters <- function() {
  allTimes <- joined[joined$X.NodeComparisons != 0, ];
  total <- length(allTimes$GroupID)
  normalStatusTimes <- allTimes[allTimes$Status == 2, ];
  
  ranges <- array();
  ranges[1] <- -Inf;
  ranges[2] <- 1000;
  ranges[3] <- 10000;
  ranges[4] <- 100000;
  ranges[5] <- Inf;
  
  for (i in 1:4) {
    inThisRange <- length(normalStatusTimes$GroupID[normalStatusTimes$totalTime / 1e6 > ranges[i] 
                                            & normalStatusTimes$totalTime / 1e6 <= ranges[i + 1]])
    cat(sprintf("%5s < time <= %5s : %6s (%6.3f%%) \r\n", ranges[i], ranges[i + 1], inThisRange, (inThisRange / total * 100)))
  }
}

plotNumberOfNodeComparisons <- function() {
  
  library(Hmisc, lattice)
  
  t1 <- data.frame(val = log10(joined$X.NodeComparisons[joined$X.NodeComparisons > 0]), method = "tool")
  t2 <- data.frame(val = log10(joined$X.StatementsInCloneFragment1[joined$X.NodeComparisons > 0] * joined$X.StatementsInCloneFragment2[joined$X.NodeComparisons > 0]), 
                   method = "exhustive")
  t3 <- rbind(t1, t2)
  
  medTool = log10(median(joined$X.NodeComparisons[joined$X.NodeComparisons > 0]))
  medExhustive = log10(median(joined$X.StatementsInCloneFragment1[joined$X.NodeComparisons > 0] * joined$X.StatementsInCloneFragment2[joined$X.NodeComparisons > 0]))
  meanTool = log10(mean(joined$X.NodeComparisons[joined$X.NodeComparisons > 0]))
  meanExhustive = log10(mean(joined$X.StatementsInCloneFragment1[joined$X.NodeComparisons > 0] * joined$X.StatementsInCloneFragment2[joined$X.NodeComparisons > 0]))
  
  
  bwplot(val ~ factor(method, labels=c("Our Approach", "Exhaustive Search")), t3,  
         panel = function(..., box.ratio) {
           panel.violin(..., col = "#cccccc", border="#777777", 
                        varwidth = FALSE, box.ratio = box.ratio, box.width=0.75)
           panel.bwplot(..., fill = "transparent", box.width=0.3, pch='|')
           panel.segments(x0=0, y0=medTool,
                          x1=0.85, y1=medTool,
                          lty=3, col="#999999")
           panel.segments(x0=2.15, y0=medExhustive,
                          x1=3, y1=medExhustive,
                          lty=3, col="#999999")
           panel.segments(x0=0, y0=meanTool+log10(1.1),
                          x1=0.85, y1=meanTool+log10(1.1),
                          lty=6, col="#999999")
           panel.segments(x0=2, y0=meanExhustive,
                          x1=3, y1=meanExhustive,
                          lty=6, col="#999999")
         },   
         axis = function(side, ...) 
         { 
           switch(side, 
                  left = { 
                    at = c(0, 1, 2, 3, 4, 5, 6, medTool, meanTool+log10(1.1));
                    lbs = c(1, 10, 100, 1000, 10000, 100000, 1000000, paste("Median=", 10^medTool, sep=""), paste("Mean=", round(10^meanTool, 2), sep=""))
                    panel.axis(side = side, outside = F,text.cex=0.7,rot=0, at = at, labels = lbs) 
                  }, 
                  right = { 
                    at = c(medExhustive, meanExhustive, 0);
                    lbs = c(paste("Median=", 10^medExhustive, sep=""), paste("Mean=", round(10^meanExhustive, 2), sep=""), "") 
                    panel.axis(side = side, outside = F,text.cex=0.7,rot=0, at = at, labels = lbs) 
                  }, 
                  axis.default(side = side, ...)) 
         },
         scales=list(
           y = list(at = c(0, 1, 2, 3, 4, 5, 6, medTool, meanTool+log10(1.1), medExhustive, meanExhustive)),
           alternating = 3
         ),
         par.settings = list(
           plot.symbol = list(pch = 1, col = "#888888"),
           box.rectangle = list(col = "#555555"),
           box.umbrella = list(col = "#555555"),
           layout.widths =
             list(left.padding = 0,
                  key.ylab.padding = 0,
                  ylab.axis.padding = 0,
                  axis.key.padding = 0,
                  right.padding = 0)
         ),
         #xlab=c("Our Approach","Brute Force"),
         ylab=""
  )
}

getStatisticalTestRestsForNodeComparisons <- function() {
  nodeComparisons <- data.frame(Tool = joined$X.NodeComparisons[joined$X.NodeComparisons > 0], 
                                Exhaustive = joined$X.StatementsInCloneFragment1[joined$X.NodeComparisons > 0] * joined$X.StatementsInCloneFragment2[joined$X.NodeComparisons > 0])
  wilcox.test(nodeComparisons$Tool, nodeComparisons$Exhaustive, alternative = "less", paired = T, conf.level = 0.95)
}



