#Author Davood Mazinanian

load <- function(path = "" ) {
  
  if (path == "") {
    path = paste(sep="/", getwd(),"CSVs");
  }
  
  readTable <- function(filePath) {
    return(read.table(filePath, sep="|", header=T, comment.char="/"));
  }
  
  getFilesList <- function(pattern) {
    return (list.files(path = path, pattern = pattern, full.names = TRUE));
  }
  
  # load CSV files from given path
  allCSVs = getFilesList("*.csv");
  for (csvFilePath in allCSVs) {
    tbl = readTable(csvFilePath);
    tableName = gsub("-", "_", substr(basename(csvFilePath), 1, nchar(basename(csvFilePath)) - 4));
    tbl$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    assign(tableName, tbl, envir = .GlobalEnv);
  }
  
  #make main reports
  reportCCfinder <<- data.frame();
  for (reportFile in getFilesList("*ccfinder*.report.csv")) {
    tempTable <- readTable(reportFile);
    tableName <- gsub("-", "_", substr(basename(reportFile), 1, nchar(basename(reportFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "ccfinder";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    reportCCfinder <<- rbind(reportCCfinder, tempTable)  
  }
  
  reportDeckard <<- data.frame();
  for (reportFile in getFilesList("*deckard*.report.csv")) {
    tempTable <- readTable(reportFile);
    tableName <- gsub("-", "_", substr(basename(reportFile), 1, nchar(basename(reportFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "deckard";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    reportDeckard <<- rbind(reportDeckard, tempTable)  
  }
  
  reportNicad <<- data.frame();
  for (reportFile in getFilesList("*nicad*.report.csv")) {
    tempTable <- readTable(reportFile);
    tableName <- gsub("-", "_", substr(basename(reportFile), 1, nchar(basename(reportFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    reportNicad <<- rbind(reportNicad, tempTable)  
  }
  
  reportCloneDR <<- data.frame();
  for (reportFile in getFilesList("*clonedr*.report.csv")) {
    tempTable <- readTable(reportFile);
    tableName <- gsub("-", "_", substr(basename(reportFile), 1, nchar(basename(reportFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "clonedr";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    reportCloneDR <<- rbind(reportCloneDR, tempTable)  
  }
  report <<- rbind(reportCCfinder, reportDeckard, reportNicad, reportCloneDR);
  
  
  #make trees tables
  treesCCFinder <<- data.frame();
  for (treesFile in getFilesList("*ccfinder*.trees.csv")) {
    tempTable <- readTable(treesFile);
    tableName <- gsub("-", "_", substr(basename(treesFile), 1, nchar(basename(treesFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "ccfinder";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    treesCCFinder <<- rbind(treesCCFinder, tempTable)  
  }
  
  treesDeckard <<- data.frame();
  for (treesFile in getFilesList("*deckard*.trees.csv")) {
    tempTable <- readTable(treesFile);
    tableName <- gsub("-", "_", substr(basename(treesFile), 1, nchar(basename(treesFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "deckard";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    treesDeckard <<- rbind(treesDeckard, tempTable)  
  }
  
  treesNicad <<- data.frame();
  for (treesFile in getFilesList("*nicad*.trees.csv")) {
    tempTable <- readTable(treesFile);
    tableName <- gsub("-", "_", substr(basename(treesFile), 1, nchar(basename(treesFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    treesNicad <<- rbind(treesNicad, tempTable)  
  }
  
  treesCloneDR <<- data.frame();
  for (treesFile in getFilesList("*clonedr*.trees.csv")) {
    tempTable <- readTable(treesFile);
    tableName <- gsub("-", "_", substr(basename(treesFile), 1, nchar(basename(treesFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "clonedr";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    treesCloneDR <<- rbind(treesCloneDR, tempTable)  
  }
  treesTable <<- rbind(treesCCFinder, treesDeckard, treesNicad, treesCloneDR);
  
  #make precondition violations tables
  
  preconditionViolationsCCFinder <<- data.frame();
  for (precondviolationFile in getFilesList("*ccfinder*.precondviolations.csv")) {
    tempTable <- readTable(precondviolationFile);
    tableName <- gsub("-", "_", substr(basename(precondviolationFile), 1, nchar(basename(precondviolationFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "ccfinder";
    preconditionViolationsCCFinder <<- rbind(preconditionViolationsCCFinder, tempTable)  
  }
  
  preconditionViolationsDeckard <<- data.frame();
  for (precondviolationFile in getFilesList("*deckard*.precondviolations.csv")) {
    tempTable <- readTable(precondviolationFile);
    tableName <- gsub("-", "_", substr(basename(precondviolationFile), 1, nchar(basename(precondviolationFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "deckard";
    preconditionViolationsDeckard <<- rbind(preconditionViolationsDeckard, tempTable)  
  }
  
  preconditionViolationsNicad <<- data.frame();
  for (precondviolationFile in getFilesList("*nicad*.precondviolations.csv")) {
    tempTable <- readTable(precondviolationFile);
    tableName <- gsub("-", "_", substr(basename(precondviolationFile), 1, nchar(basename(precondviolationFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    preconditionViolationsNicad <<- rbind(preconditionViolationsNicad, tempTable)  
  }
  
  preconditionViolationsCloneDR <<- data.frame();
  for (precondviolationFile in getFilesList("*clonedr*.precondviolations.csv")) {
    tempTable <- readTable(precondviolationFile);
    tableName <- gsub("-", "_", substr(basename(precondviolationFile), 1, nchar(basename(precondviolationFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "clonedr";
    preconditionViolationsCloneDR <<- rbind(preconditionViolationsCloneDR, tempTable)  
  }
  
  preconditionViolations <<- rbind(preconditionViolationsCCFinder, preconditionViolationsDeckard, preconditionViolationsNicad, preconditionViolationsCloneDR);
  
  joined <<- merge(report, treesTable, by=c("GroupID", "PairID", "projectName"))
  joined$totalTime <<- joined$SubtreeMatchingWallNanoTime + joined$PDGMappingWallNanoTime
  joined$average <<- (joined$X.StatementsInCloneFragment1 + joined$X.StatementsInCloneFragment2) / 2
  
  
  joinedCCFinder <<- merge (reportCCfinder, treesCCFinder)
  joinedCCFinder$totalTime <<- joinedCCFinder$SubtreeMatchingWallNanoTime + joinedCCFinder$PDGMappingWallNanoTime
  joinedCCFinder$average <<- (joinedCCFinder$X.StatementsInCloneFragment1 + joinedCCFinder$X.StatementsInCloneFragment2) / 2
  
  joinedDeckard <<- merge (reportDeckard, treesDeckard)
  joinedDeckard$totalTime <<- joinedDeckard$SubtreeMatchingWallNanoTime + joinedDeckard$PDGMappingWallNanoTime
  joinedDeckard$average <<- (joinedDeckard$X.StatementsInCloneFragment1 + joinedDeckard$X.StatementsInCloneFragment2) / 2
  
  joinedNicad <<- merge (reportNicad, treesNicad)
  joinedNicad$totalTime <<- joinedNicad$SubtreeMatchingWallNanoTime + joinedNicad$PDGMappingWallNanoTime
  joinedNicad$average <<- (joinedNicad$X.StatementsInCloneFragment1 + joinedNicad$X.StatementsInCloneFragment2) / 2
  
  joinedCloneDR <<- merge (reportCloneDR, treesCloneDR)
  joinedCloneDR$totalTime <<- joinedCloneDR$SubtreeMatchingWallNanoTime + joinedCloneDR$PDGMappingWallNanoTime
  joinedCloneDR$average <<- (joinedCloneDR$X.StatementsInCloneFragment1 + joinedCloneDR$X.StatementsInCloneFragment2) / 2
}