loadConsistent <- function() {
  
  path = paste(sep="/", getwd(),"Nicad-Consistent-CSVs");
  
  
  readTable <- function(filePath) {
    return(read.table(filePath, sep="|", header=T, comment.char="/"));
  }
  
  getFilesList <- function(pattern) {
    return (list.files(path = path, pattern = pattern, full.names = TRUE));
  }
  
  # load CSV files from given path
  allCSVs = getFilesList("*.csv");
  for (csvFilePath in allCSVs) {
    tbl = readTable(csvFilePath);
    tableName = gsub("-", "_", substr(basename(csvFilePath), 1, nchar(basename(csvFilePath)) - 4));
    tbl$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    assign(tableName, tbl, envir = .GlobalEnv);
  }
  
  reportNicadConsistent <<- data.frame();
  for (reportFile in getFilesList("*.report.csv")) {
    tempTable <- readTable(reportFile);
    tableName <- gsub("-", "_", substr(basename(reportFile), 1, nchar(basename(reportFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    reportNicadConsistent <<- rbind(reportNicadConsistent, tempTable)  
  }
  
  treesNicadConsistent <<- data.frame();
  for (treesFile in getFilesList("*.trees.csv")) {
    tempTable <- readTable(treesFile);
    tableName <- gsub("-", "_", substr(basename(treesFile), 1, nchar(basename(treesFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    tempTable$ID <- paste(tempTable$GroupID, tempTable$PairID, sep="-")
    treesNicadConsistent <<- rbind(treesNicadConsistent, tempTable)  
  }
  
  preconditionViolationsNicadConsistent <<- data.frame();
  for (precondviolationFile in getFilesList("*.precondviolations.csv")) {
    tempTable <- readTable(precondviolationFile);
    tableName <- gsub("-", "_", substr(basename(precondviolationFile), 1, nchar(basename(precondviolationFile)) - 4));
    tempTable$projectName <- gsub(".report|.trees|.precondviolations", "", tableName);
    tempTable$tool <- "nicad";
    preconditionViolationsNicadConsistent <<- rbind(preconditionViolationsNicadConsistent, tempTable)  
  }
  
  joinedNicadConsistent <<- merge (reportNicadConsistent, treesNicadConsistent)
  joinedNicadConsistent$totalTime <<- joinedNicadConsistent$SubtreeMatchingWallNanoTime + joinedNicadConsistent$PDGMappingWallNanoTime
  joinedNicadConsistent$average <<- (joinedNicadConsistent$X.StatementsInCloneFragment1 + joinedNicadConsistent$X.StatementsInCloneFragment2) / 2
  
}

getConsistentResults <- function() {
  
  loadConsistent()
  
  cat("Nicad - Consistent results\n")
  all = length(joinedNicadConsistent$GroupID[joinedNicadConsistent$X.NodeComparisons > 0])
  nicadRefactorable = length(joinedNicadConsistent$GroupID[joinedNicadConsistent$X.PreconditionViolations == 0 & joinedNicadConsistent$X.NodeComparisons > 0 & joinedNicadConsistent$X.MappedStatements > 0])
  cat(sprintf("All: %s \t Refactorable: %s (%s%%)\n", all, nicadRefactorable, nicadRefactorable / all * 100))
  
  for (i in 1:4) {
    allOfThisType <- length(joinedNicadConsistent$GroupID[joinedNicadConsistent$CloneType == i & joinedNicadConsistent$X.NodeComparisons > 0])
    refactorableThisType <- length(joinedNicadConsistent$GroupID[joinedNicadConsistent$X.PreconditionViolations == 0 & joinedNicadConsistent$CloneType == i & joinedNicadConsistent$X.NodeComparisons > 0 & joinedNicadConsistent$X.MappedStatements > 0])
    cat(sprintf("Clone Type: %1s\tTotal: %6s (%4s)\tRefactorable: %5s (%4s) \r\n",
                i, allOfThisType, round(allOfThisType/all,3)*100, refactorableThisType, round(refactorableThisType / allOfThisType, 3) * 100))
  }
   
}

test <- function() {
  reportfree <- rbind(jfreechart_1.0.10_ccfinder.trees, jfreechart_1.0.10_clonedr.trees, jfreechart_1.0.10_deckard.trees, jfreechart_1.0.10_nicad.trees)
  treesfree <- rbind(jfreechart_1.0.10_ccfinder.report, jfreechart_1.0.10_clonedr.report, jfreechart_1.0.10_deckard.report, jfreechart_1.0.10_nicad.report)
  t <- merge(reportfree, treesfree, by=c("projectName", "GroupID", "PairID"))
  t <- t[t$X.PreconditionViolations == 0 & t$X.NodeComparisons > 0 & t$X.MappedStatements > 0 & t$CloneType == 3, ]
  head(t[with(t, order(-t$X.StatementsInCloneFragment1)), ], n=20)
}

