#Author Davood Mazinanian

getViolationNames <- function() {
  return (c(
    "EXPRESSION_DIFFERENCE_CANNOT_BE_PARAMETERIZED",
    "EXPRESSION_DIFFERENCE_IS_FIELD_UPDATE",
    "EXPRESSION_DIFFERENCE_IS_VOID_METHOD_CALL",
    "EXPRESSION_DIFFERENCE_IS_METHOD_CALL_THROWING_EXCEPTION_WITHIN_MATCHED_TRY_BLOCK",
    "INFEASIBLE_UNIFICATION_DUE_TO_VARIABLE_TYPE_MISMATCH",
    "INFEASIBLE_UNIFICATION_DUE_TO_MISSING_MEMBERS_IN_THE_COMMON_SUPERCLASS",
    "UNMATCHED_STATEMENT_CANNOT_BE_MOVED_BEFORE_OR_AFTER_THE_EXTRACTED_CODE",
    "UNMATCHED_STATEMENT_CANNOT_BE_MOVED_BEFORE_THE_EXTRACTED_CODE_DUE_TO_CONTROL_DEPENDENCE",
    "UNMATCHED_BREAK_STATEMENT",
    "UNMATCHED_CONTINUE_STATEMENT",
    "UNMATCHED_RETURN_STATEMENT",
    "UNMATCHED_THROW_STATEMENT",
    "UNMATCHED_EXCEPTION_THROWING_STATEMENT_NESTED_WITHIN_MATCHED_TRY_BLOCK",
    "MULTIPLE_RETURNED_VARIABLES",
    "UNEQUAL_NUMBER_OF_RETURNED_VARIABLES",
    "SINGLE_RETURNED_VARIABLE_WITH_DIFFERENT_TYPES",
    "BREAK_STATEMENT_WITHOUT_LOOP",
    "CONTINUE_STATEMENT_WITHOUT_LOOP",
    "CONDITIONAL_RETURN_STATEMENT",
    "MULTIPLE_UNMATCHED_STATEMENTS_UPDATE_THE_SAME_VARIABLE"
  ))
}

getPreconditionViolations <- function() {
  
  violationNames = getViolationNames()
  
  getPercentagesMatrix = function() {
    tools = unique(preconditionViolations$tool)
    toReturn = matrix(nrow = length(violationNames) * length(tools), ncol = 3)
    i = 0
    for (tool in tools) {
      table = preconditionViolations[preconditionViolations$tool == tool,]
      allPrecondsInThisTool = length(table$GroupID)
      i = i + 1
      for (k in 0:(length(violationNames) - 1)) {
        countThisType = length(table$GroupID[table$PreconditionViolationType == k])        
        ratioThisType =  countThisType / allPrecondsInThisTool
        rowIndex = ((i-1) * length(violationNames)) + k + 1;
        toReturn[rowIndex, 1] = violationNames[k + 1]
        toReturn[rowIndex, 2] = tool
        toReturn[rowIndex, 3] = countThisType #100*round(ratioThisType,3)
        #cat(sprintf("%87s : %5s (%4s%%)\r\n",
        #            violationNames[i], 
        #            countThisType,
        #            100 * round(ratioThisType, 3)
        #            ))
        
      }
    }
    
    return (toReturn)
  }
  
  df = data.frame(getPercentagesMatrix())
  colnames(df) = c("Precondition", "Tool", "Percentage")
  #return(df)
  
  #bwplot(factor(df$Precondition, labels=1:20) ~ df$Percentage, groups =df$Tool, panel = panel.barchart)
  #return (df)
  
  df2 = data.frame(Precondition = unique(df$Precondition))
  for (precondition in unique(df2$Precondition)) {
    for (tool in df$Tool) {
      df2[df2$Precondition == precondition, tool] <- df$Percentage[df$Tool == tool & df$Precondition == precondition]
    }
  }
  colnames(df2) <- c("Precondition", "CCFinder", "Deckard", "Nicad", "CloneDR")
  return (df2)
}

preconditionViolationsPerType <- function() {
  mrg <- merge(preconditionViolations, treesTable, by = c("GroupID", "PairID", "TreeID", "projectName"))
  cat(length(mrg$GroupID[mrg$CloneType == 1 & mrg$PreconditionViolationType == 0]))
  #test <- aggregate(mrg, by=list("PreconditionViolationType", "CloneType"))
  #cat(colnames(test))
  #violationNames <- getViolationNames()
  #df = data.frame(Precondition = violationNames, TypeI = 0, TypeII = 0, TypeIII = 0, Unknown = 0)
  #for (k in 0:(length(violationNames) - 1)) {
  #  precondName <- violationNames[k + 1]
  #  df$TypeI[df$Precondition == precondName] = length(mrg$GroupID[mrg$CloneType == 1 & mrg$PreconditionViolationType == k])
  # df$TypeII[df$Precondition == precondName] = length(mrg$GroupID[mrg$CloneType == 2 & mrg$PreconditionViolationType == k])
  # df$TypeIII[df$Precondition == precondName] = length(mrg$GroupID[mrg$CloneType == 3 & mrg$PreconditionViolationType == k])
  #  df$Unknown[df$Precondition == precondName] = length(mrg$GroupID[mrg$CloneType == 4 & mrg$PreconditionViolationType == k])
  #}
  #return(df)
}

getPreconditionViolationsPlots<-function() {
  bwplot(log10(joined$X.PreconditionViolations[joined$X.PreconditionViolations > 0]) ~ factor(joined$tool.x), panel = function(..., box.ratio) {
      panel.violin(..., col = "#cccccc", border="#777777", varwidth = FALSE, box.ratio = box.ratio, box.width=0.7)
      panel.bwplot(..., fill = "transparent", box.width=0.3, pch='|')
    }
  )
}

