#Author Davood Mazinanian
printTestCodeResultsTable <- function() {
  printMainReportTable(2)
}

printProductionCodeResultsTable <- function() {
  printMainReportTable(1)
}

printAllCodeResultsTable <- function() {
  printMainReportTable(0)
}


printMainReportTable <- function(mode) {
  
  getClonePairsAndRefactoring <- function(prj, tableCCFinder, tableDeckard, tableCloneDR, tableNicad) {
    ccfinder <- getValues(tableCCFinder, mode);
    deckard <- getValues(tableDeckard, mode);
    clonedr <- getValues(tableCloneDR, mode);
    nicad <- getValues(tableNicad, mode);
    cat(sprintf(getRowFormat(), prj, 
                ccfinder[1], 100*round(ccfinder[1]/ccfinder[3], 3), ccfinder[2], 100 * round(ccfinder[2] / ccfinder[1], 3),
                deckard[1], 100*round(deckard[1]/deckard[3], 3), deckard[2], 100 * round(deckard[2] / deckard[1], 3),
                clonedr[1], 100*round(clonedr[1]/clonedr[3], 3), clonedr[2], 100 * round(clonedr[2] / clonedr[1], 3),
                nicad[1], 100*round(nicad[1]/nicad[3], 3), nicad[2], 100 * round(nicad[2] / nicad[1], 3)
                )
        );

  }
  
  getValues <- function(table, mode) {
    toReturn <- array()
    toReturn[3] <- length(table$GroupID[table$X.NodeComparisons > 0]);
    if (mode == 0) {
      toReturn[1] <- toReturn[3]
      toReturn[2] <- length(table$GroupID[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0]);
    } else if (mode == 1) {
      toReturn[1] <- length(table$GroupID[table$IsTestCode < 3 & table$X.NodeComparisons > 0]);
      toReturn[2] <- length(table$GroupID[table$X.PreconditionViolations == 0 & table$IsTestCode < 3 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0]);
    } else if (mode == 2) {
      toReturn[1] <- length(table$GroupID[table$IsTestCode == 3 & table$X.NodeComparisons > 0]);
      toReturn[2] <- length(table$GroupID[table$X.PreconditionViolations == 0 & table$IsTestCode == 3 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0]);
    }
    return(toReturn);
  }
  
  getRowFormat <- function() {
    return("%31s & %7s (%4s) & %7s (%4s) &  %7s (%4s) &  %7s (%4s) &  %7s (%4s) &  %7s (%4s) &  %7s (%4s) &  %7s (%4s) \\\\ \r\n");
  }
  
  cat(sprintf("\r\n%56s%37s%37s%37s\r\n", "CCFinder", "Deckard", "CloneDR", "Nicad"));
  #cat(sprintf(getRowFormat(), "Project", "All", "", "Refac", "", "All", "", "Refac", "", "All", "", "Refac", ""));
  
  tableArray <- unique(gsub("_ccfinder|_deckard|_nicad|_clonedr", "", joined$projectName));
  for (tableName in tableArray)
    #if (tableName != "emf_2.4.1" && tableName != "jEdit_4.2")
    getClonePairsAndRefactoring(tableName, 
      joinedCCFinder[joinedCCFinder$projectName == paste(tableName,"ccfinder", sep="_"),],
      joinedDeckard[joinedDeckard$projectName == paste(tableName,"deckard", sep="_"),],
      joinedCloneDR[joinedCloneDR$projectName == paste(tableName,"clonedr", sep="_"),],
      joinedNicad[joinedNicad$projectName == paste(tableName,"nicad", sep="_"),])
    
  ccfinderAllSum = getValues(joinedCCFinder, mode);
  deckardAllSum = getValues(joinedDeckard, mode);
  clonedrAllSum = getValues(joinedCloneDR, mode);
  nicadAllSum = getValues(joinedNicad, mode);

  
  cat(sprintf(getRowFormat(), "", 
              ccfinderAllSum[1], 100 * round(ccfinderAllSum[1] / ccfinderAllSum[3], 3), ccfinderAllSum[2], 100 * round(ccfinderAllSum[2] / ccfinderAllSum[1], 3),
              deckardAllSum[1], 100 * round(deckardAllSum[1] / deckardAllSum[3], 3), deckardAllSum[2], 100 * round(deckardAllSum[2] / deckardAllSum[1], 3), 
              clonedrAllSum[1], 100 * round(clonedrAllSum[1] / clonedrAllSum[3], 3), clonedrAllSum[2], 100 * round(clonedrAllSum[2] / clonedrAllSum[1], 3), 
            nicadAllSum[1], 100 * round(nicadAllSum[1] / nicadAllSum[3], 3), nicadAllSum[2], 100 * round(nicadAllSum[2]/ nicadAllSum[1], 3)));
  
}



testVSProduction <- function() {
 
  getInfo = function(prj, table) {
    
    prod = length(table$ID[table$IsTestCode != 3 & table$X.NodeComparisons > 0])
    prodRefactorable = length(table$ID[table$IsTestCode != 3 & table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0])
 
    test = length(table$ID[table$IsTestCode == 3 & table$X.NodeComparisons > 0])
    testRefactorable = length(table$ID[table$IsTestCode == 3 & table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0])

    cat(sprintf("%s production refactorable ratio: %s, test refactorable ratio: %s\r\n", 
                prj, 
                100 * round(prodRefactorable / prod, 3), 
                100 * round(testRefactorable / test, 3)))
  }
  
  getInfo("CCFinder", joinedCCFinder)
  getInfo("Deckard", joinedDeckard)
  getInfo("CloneDR", joinedCloneDR)
  getInfo("Nicad", joinedNicad)
}

printRefactorableLocation <- function(mode = "") {

  locationRefactorability <- function(table, name) {
    
    if (mode == "test")
      table <- table[table$IsTestCode == 3,]
    else if (mode == "production")
      table <- table[table$IsTestCode < 3,]
    
    names <- array();
    names[1] <- "Same method"
    names[2] <- "Same declaring class"
    names[3] <- "Same java file"
    names[4] <- "Same hierarchy"
    names[5] <- "Different classes"
    #SAME_METHOD  
    #SAME_DECLARING_CLASS  
    #SAME_JAVA_FILE  
    #SAME_HIERARCHY  
    #DIFFERENT_CLASSES
    
    totalThisTool <- length(table$ID[table$X.NodeComparisons > 0]);
    
    for (i in 0:4) {
      refactorableNumberOfCases <- length(table$ID[table$ClonePairLocation == i & table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0]);
      totalCases <- length(table$ID[table$ClonePairLocation == i & table$X.NodeComparisons > 0]);
      ratio <-  refactorableNumberOfCases / totalCases;
      cat(sprintf("%10s - %20s Total: %6s (%4s%%) Refactorable: %6s (%4s%%)\r\n",
                  name, names[i+1], totalCases, 100 * round(totalCases / totalThisTool, 3), refactorableNumberOfCases, 100 * round(ratio, 3)))
    }    
  }
  
  #locationRefactorability(joinedCCFinder[joinedCCFinder$IsTestCode == 3,], "CCFinder")
  locationRefactorability(joinedCCFinder, "CCFinder")
  cat("\r\n")
  locationRefactorability(joinedDeckard, "Deckard")
  cat("\r\n")
  locationRefactorability(joinedCloneDR, "CloneDr")
  cat("\r\n")
  locationRefactorability(joinedNicad, "NiCad")
}

printRefactorableCloneType <- function(mode = "") {
  
  printRefactorableCloneTypeTable <- function(table) {
    
    if (mode == "test")
      table <- table[table$IsTestCode == 3,]
    else if (mode == "production")
      table <- table[table$IsTestCode < 3,]
    
    allTreesLength <- length(table$GroupID[table$X.NodeComparisons > 0]);
    for (i in 1:4) {
      allOfThisType <- length(table$GroupID[table$CloneType == i & table$X.NodeComparisons > 0])
      refactorableThisType <- length(table$GroupID[table$X.PreconditionViolations == 0 & table$CloneType == i & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0])
      cat(sprintf("Clone Type: %1s\tTotal: %6s (%4s)\tRefactorable: %5s (%4s) \r\n",
                  i, allOfThisType, round(allOfThisType/allTreesLength,3)*100, refactorableThisType, round(refactorableThisType / allOfThisType, 3) * 100))
    }
  }
  
  cat("CCFinder\r\n")
  printRefactorableCloneTypeTable(joinedCCFinder)
  cat("\r\nDeckard\r\n")
  printRefactorableCloneTypeTable(joinedDeckard)
  cat("\r\nCloneDr\r\n")
  printRefactorableCloneTypeTable(joinedCloneDR)
  cat("\r\nNicad\r\n")
  printRefactorableCloneTypeTable(joinedNicad)
  

}

getRefactorableMethodClones <- function() {

  refactorableMethods <- function(tool = "") {
    if (tool == "") {
      table <- joined;
      tool = "entire dataset"
    } else {
      table <- joined[joined$tool.x == tool, ]
    }
    table$refactorable <- "Non-refactorable"
    table$refactorable[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0] <- "Refactorable"
    
    methodClones <- table[table$X.NodeComparisons > 0 & 
            table$X.PDGNodesInMethod1 == table$X.StatementsInCloneFragment1 & 
            table$X.PDGNodesInMethod2 == table$X.StatementsInCloneFragment2, ]
    
    method = nrow(methodClones)
    methodRatio = round(method / nrow(table[table$X.NodeComparisons > 0,]), 3) * 100
    refac = nrow(methodClones[methodClones$refactorable == "Refactorable", ])
    cat(sprintf("%s: Method clones: %s (%s%%) | Refactorable: %s (%s%%) \n", tool, method, methodRatio, refac, round(refac/method,3)*100))
    
  }
  
  refactorableMethods()
  refactorableMethods("ccfinder")
  refactorableMethods("deckard")
  refactorableMethods("clonedr")
  refactorableMethods("nicad")
  
}

test2 <- function(table) {
  allTreesLength <- length(table$GroupID[table$X.NodeComparisons > 0]);
  for (i in 1:4) {
    allOfThisType <- length(table$GroupID[table$CloneType == i & table$X.NodeComparisons > 0])
    cat(sprintf("Clone Type: %1s\tTotal: %6s (%4s) \r\n",
                i, allOfThisType, round(allOfThisType/allTreesLength,3)*100 ))
  }
}