#Author Davood Mazinanian

plotSizes <- function() {
    
  library(Hmisc, lattice)
  
  
  medCCFinder = log10(median(joinedCCFinder$average))
  meanCCFinder = log10(mean(joinedCCFinder$average))
  
  medDeckard = log10(median(joinedDeckard$average))
  meanDeckard = log10(mean(joinedDeckard$average))
  
  medCloneDr  = log10(median(joinedCloneDR$average))
  meanCloneDr = log10(mean(joinedCloneDR$average))
  
  medNicad = log10(median(joinedNicad$average))
  meanNicad = log10(mean(joinedNicad$average))
  
  
  bwplot(log10(joined$average) ~ factor(joined$tool.x, levels=c("ccfinder", "deckard", "clonedr", "nicad"), labels=c("CCFinder", "Deckard", "CloneDR", "NiCad")),  
         panel = function(..., box.ratio) {
           panel.violin(..., col = "#cccccc", border="#777777", varwidth = FALSE, box.ratio = box.ratio, box.width=0.7)
           panel.bwplot(..., fill = "transparent", box.width=0.3, pch='|')
           
           panel.segments(x0=1.15, y0=medCCFinder, x1=1.45, y1=medCCFinder, lty=3, col="#999999")
           panel.segments(x0=0, y0=meanCCFinder, x1=1, y1=meanCCFinder, lty=6, col="#999999")
           panel.text(x=1.45 + 0.15, y=medCCFinder-0.03, labels=paste("Median=", 10^medCCFinder, sep=""), cex=0.60)
           
           panel.segments(x0=2.15, y0=medDeckard, x1=2.45, y1=medDeckard, lty=3, col="#999999")
           panel.segments(x0=1.55, y0=meanDeckard, x1=2, y1=meanDeckard, lty=6, col="#999999")
           panel.text(x=2.45, y=medDeckard + 0.03, labels=paste("Median=", 10^medDeckard,sep=""), cex=0.60)
           panel.text(x=1.55, y=meanDeckard + 0.03, labels=paste("Mean=", round(10^meanDeckard, 1), sep=""), cex=0.60)
           
           panel.segments(x0=3, y0=medCloneDr, x1=3.45, y1=medCloneDr, lty=3, col="#999999")
           panel.segments(x0=2.55, y0=meanCloneDr, x1=3, y1=meanCloneDr, lty=6, col="#999999")
           panel.text(x=3.45, y=medCloneDr-0.03, labels=paste("Median=", 10^medCloneDr,sep=""), cex=0.60)
           panel.text(x=2.55, y=meanCloneDr-0.03, labels=paste("Mean=", round(10^meanCloneDr, 1),sep=""), cex=0.60)
           
           panel.segments(x0=3.55, y0=medNicad, x1=3.85, y1=medNicad, lty=3, col="#999999")
           panel.segments(x0=4, y0=meanNicad, x1=5, y1=meanNicad, lty=6, col="#999999")
           panel.text(x=3.55, y=medNicad-0.03, labels=paste("Median=", 10^medNicad, sep=""), cex=0.60)
         },   
         axis = function(side, ...) { 
           switch(side, 
                  left = { 
                    at = c(0, log10(2), log10(5), log10(20), log10(35), 2, log10(300), medCCFinder, meanCCFinder);
                    lbs = c(1, 2, 5, 20, 35, 100, 300, "", paste("Mean=", round(10^meanCCFinder, 1), sep=""))
                    panel.axis(side = side, outside = F,text.cex=0.6,rot=0, at = at, labels = lbs) 
                  }, 
                  right = { 
                    at = c(medNicad, meanNicad, 0);
                    lbs = c("", paste("Mean=", round(10^meanNicad, 1), sep=""), "") 
                    panel.axis(side = side, outside = F,text.cex=0.6,rot=0, at = at, labels = lbs) 
                  }, 
                  axis.default(side = side, ...)) 
         }, 
         scales = list(
           y = list(at = c(0), labels = c("                 "), cex=0.7),
           alternating = 3
         ),
         par.settings = list(
           plot.symbol = list(pch = 1, col = "#888888"),
           box.rectangle = list(col = "#555555"),
           box.umbrella = list(col = "#555555"),
           layout.widths =
             list(left.padding = 0,
                  key.ylab.padding = 0,
                  ylab.axis.padding = 0,
                  axis.key.padding = 0,
                  right.padding = 0)
           #,layout.widths=list(right.padding=+5)
         ),
         ylab = ""
  )
}

plotSizeRefactorable <- function() {
  meanLty = 6
  medianLty = 3
  library(lattice) 
  #library(latticeExtra) 
  table <- joined;
  table$averageSize[table$X.NodeComparisons > 0] <- (table$X.StatementsInCloneFragment1[table$X.NodeComparisons > 0] + table$X.StatementsInCloneFragment2[table$X.NodeComparisons > 0]) / 2
  table$refactorable <- "Non-refactorable"
  table$refactorable[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0] <- "Refactorable"
  bwplot(
    table$averageSize ~ table$refactorable | ordered(table$tool.x, levels=c("ccfinder", "deckard", "clonedr", "nicad"), labels=c("CCFinder", "Deckard", "CloneDR", "Nicad")), 
    panel = function(..., box.ratio) {
      
      panel.violin(..., col = "#cccccc", border="#777777", varwidth = FALSE, box.ratio = box.ratio, box.width=0.75)
      panel.bwplot(..., fill = "transparent", box.width=0.3, pch='|')
      if (panel.number() == 1) {
        
        ccfinder = table[table$X.NodeComparisons > 0 & table$tool.x == "ccfinder", ]
        
        ccfinderNR = ccfinder$averageSize[ccfinder$refactorable == "Non-refactorable"]
        ccfinderNRMed = median(ccfinderNR)
        ccfinderNRMean = mean(ccfinderNR)
        panel.segments(x0 = 0, y0 = log10(ccfinderNRMed), x1 = .85, y1 = log10(ccfinderNRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 0, y0 = log10(ccfinderNRMean - 1), x1 = .85, y1 = log10(ccfinderNRMean - 1), lty = meanLty, col="#999999")
        
        ccfinderR = ccfinder$averageSize[ccfinder$refactorable == "Refactorable"]
        ccfinderRMed = median(ccfinderR)
        ccfinderRMean = mean(ccfinderR)
        panel.segments(x0 = 2.15, y0 = log10(ccfinderRMed), x1 = 3, y1 = log10(ccfinderRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 2, y0 = log10(ccfinderRMean), x1 = 3, y1 = log10(ccfinderRMean), lty = meanLty, col="#999999")
        
      } else if (panel.number() == 2) {
        deckard = table[table$X.NodeComparisons > 0 & table$tool.x == "deckard", ]
        
        deckardNR = deckard$averageSize[deckard$refactorable == "Non-refactorable"]
        deckardNRMed = median(deckardNR)
        deckardNRMean = mean(deckardNR)
        panel.segments(x0 = 0, y0 = log10(deckardNRMed), x1 = .85, y1 = log10(deckardNRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 0, y0 = log10(deckardNRMean + .3), x1 = .85, y1 = log10(deckardNRMean + .3), lty = meanLty, col="#999999")
        
        deckardR = deckard$averageSize[deckard$refactorable == "Refactorable"]
        deckardRMed = median(deckardR)
        deckardRMean = mean(deckardR)
        panel.segments(x0 = 2.15, y0 = log10(deckardRMed), x1 = 3, y1 = log10(deckardRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 2, y0 = log10(deckardRMean), x1 = 3, y1 = log10(deckardRMean), lty = meanLty, col="#999999")
        
      } else if (panel.number() == 3) {
        nicad = table[table$X.NodeComparisons > 0 & table$tool.x == "nicad", ]
        
        nicadNR = nicad$averageSize[nicad$refactorable == "Non-refactorable"]
        nicadNRMed = median(nicadNR)
        nicadNRMean = mean(nicadNR)
        panel.segments(x0 = 0, y0 = log10(nicadNRMed), x1 = .85, y1 = log10(nicadNRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 0, y0 = log10(nicadNRMean), x1 = .85, y1 = log10(nicadNRMean), lty = meanLty, col="#999999")
        
        nicadR = nicad$averageSize[nicad$refactorable == "Refactorable"]
        nicadRMed = median(nicadR)
        nicadRMean = mean(nicadR)
        panel.segments(x0 = 2.15, y0 = log10(nicadRMed), x1 = 3, y1 = log10(nicadRMed), lty = medianLty, col="#999999")
        panel.segments(x0 = 2, y0 = log10(nicadRMean), x1 = 3, y1 = log10(nicadRMean), lty = meanLty, col="#999999")
      }
    }, 
    axis = function(side, ...) 
    { 
      if (panel.number() == 1) {
        
        ccfinder = table[table$X.NodeComparisons > 0 & table$tool.x == "ccfinder", ]
        
        switch(side, 
           left = { 
             ccfinderNR = ccfinder$averageSize[ccfinder$refactorable == "Non-refactorable"]
             ccfinderNRMed = median(ccfinderNR)
             ccfinderNRMean = mean(ccfinderNR)
             at = c(0, log10(2), log10(20), log10(50), 2, log10(ccfinderNRMed), log10(ccfinderNRMean - 1));
             lbs = c(1, 2, 20, 50, 100, paste("Median=", ccfinderNRMed, sep=""), paste("Mean=", round(ccfinderNRMean, 1), sep=""))
             panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs) 
           }, 
           right = { 
             ccfinderR = ccfinder$averageSize[ccfinder$refactorable == "Refactorable"]
             ccfinderRMed = median(ccfinderR)
             ccfinderRMean = mean(ccfinderR)
             atl = c(log10(ccfinderRMed), log10(ccfinderRMean), 0);
             lbs = c(paste("Median=", ccfinderRMed, sep=""), paste("Mean=", round(ccfinderRMean, 1), sep=""), "")
             panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = atl, labels = lbs)
           }, 
           axis.default(side = side, ...)) 
        
      } else if (panel.number() == 2) {
        
        deckard = table[table$X.NodeComparisons > 0 & table$tool.x == "deckard", ]
        
        switch(side, 
               left = { 
                 deckardNR = deckard$averageSize[deckard$refactorable == "Non-refactorable"]
                 deckardNRMed = median(deckardNR)
                 deckardNRMean = mean(deckardNR)
                 at = c(0, log10(2), log10(20), log10(50), 2, log10(deckardNRMed), log10(deckardNRMean + .3))
                 lbs = c(1, 2, 20, 50, 100, paste("Median=", round(deckardNRMed, 1), sep=""), paste("Mean=", round(deckardNRMean, 1), sep=""))
                 panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs) 
               }, 
               right = { 
                 deckardR = deckard$averageSize[deckard$refactorable == "Refactorable"]
                 deckardRMed = median(deckardR)
                 deckardRMean = mean(deckardR)
                 at = c(log10(deckardRMed), log10(deckardRMean), 0);
                 lbs = c(paste("Median=", deckardRMed, sep=""), paste("Mean=", round(deckardRMean, 1), sep=""), "")
                 panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs)                
               }, 
               axis.default(side = side, ...)) 
        
      } else if (panel.number() == 3) {
        nicad = table[table$X.NodeComparisons > 0 & table$tool.x == "nicad", ]
        
        switch(side, 
               left = { 
                 nicadNR = nicad$averageSize[nicad$refactorable == "Non-refactorable"]
                 nicadNRMed = median(nicadNR)
                 nicadNRMean = mean(nicadNR)
                 at = c(0, log10(2), log10(20), log10(50), 2, log10(nicadNRMed), log10(nicadNRMean))
                 lbs = c(1, 2, 20, 50, 100, paste("Median=", nicadNRMed, sep=""), paste("Mean=", round(nicadNRMean, 1), sep=""))
                 panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs) 
               }, 
               right = { 
                 nicadR = nicad$averageSize[nicad$refactorable == "Refactorable"]
                 nicadRMed = median(nicadR)
                 nicadRMean = mean(nicadR)
                 at = c(log10(nicadRMed), log10(nicadRMean), 0);
                 lbs = c(paste("Median=", nicadRMed, sep=""), paste("Mean=", round(nicadRMean, 1), sep=""), "")
                 panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs)                
               }, 
               axis.default(side = side, ...)) 
      }
    },
    layout=(c(1,3)),
    scales = list(
      y = list(log=T)
    ),
    par.settings = list(
      strip.background=list(col="#dddddd"),
      layout.heights = list(strip = 1.4),
      plot.symbol = list(pch = 1, col = "#888888"),
      box.rectangle = list(col = "#555555"),
      box.umbrella = list(col = "#555555"),
      layout.widths =
        list(left.padding = 0,
             key.ylab.padding = 0,
             ylab.axis.padding = 0,
             axis.key.padding = 0,
             right.padding = 0)
      #,layout.widths=list(right.padding=+5)
    ),
    ylab = ""
  )
}

#library(vioplot)
#length(table$GroupID[table$averageSize >= 2.5 & table$averageSize <= 7.5 & table$X.PreconditionViolations > 0]) / length(table$GroupID[table$X.PreconditionViolations > 0])
#vioplot(table$averageSize[table$averageSize >= 2.5 &table$averageSize <= 7.5 & table$X.PreconditionViolations > 0])

plotECDFs <- function() {
  
  plotECDF <- function(deck) {
    table$averageSize[table$X.NodeComparisons > 0] <- getSize(deck)
    plot(ecdf(table$averageSize[table$X.PreconditionViolations == 0 & table$averageSize < 10 ]))
    plot(ecdf(table$averageSize[table$X.PreconditionViolations > 0 & table$averageSize < 10 ]))
  }
  
  par(mfrow=c(3,2))
  plotECDF(joinedCCFinder)
  plotECDF(joinedDeckard)
  plotECDF(joinedNicad)
  par(mfrow=c(1,1))
}

getStatisticalTestResultsForAverageSize <- function() {
  table <- joined;
  table$refactorable <- "Non-refactorable"
  table$refactorable[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0] <- "Refactorable"
  
  getTest <- function(tool) {
    nonRefactorable <- table$average[table$X.NodeComparisons > 0 & table$tool.x == tool & table$refactorable == "Non-refactorable"]
    refactorable <-    table$average[table$X.NodeComparisons > 0 & table$tool.x == tool & table$refactorable == "Refactorable"]
    print(tool)
    t <- wilcox.test(refactorable, nonRefactorable, alternative = "less", conf.level = 0.95, paired = F)
    print(t)
  }
  
  getTest("ccfinder")
  getTest("deckard")
  getTest("clonedr")
  getTest("nicad")
  
}

getStatisticalTestsForSizeDifferencesTools <- function() {
  table <- joined;
  table$refactorable <- "Non-refactorable"
  table$refactorable[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0] <- "Refactorable"

  getTest <- function(tool1, tool2) {
    refactorable1 <- table$average[table$X.NodeComparisons > 0 & table$tool.x == tool1 & table$refactorable == "Refactorable"]
    refactorable2 <- table$average[table$X.NodeComparisons > 0 & table$tool.x == tool2 & table$refactorable == "Refactorable"]
    t <- wilcox.test(refactorable1, refactorable2, alternative = "greater", conf.level = 0.95, paired = F)
    print(t)
  }

  getTest("deckard", "ccfinder")
  getTest("deckard", "clonedr")
  getTest("deckard", "nicad")
  getTest("ccfinder", "clonedr")
  getTest("ccfinder", "nicad")
  getTest("nicad", "clonedr")
  
}



plotSizeRefactorableSeparate <- function(table) {
  library(lattice)
  
  meanLty = 6
  medianLty = 3
  
  table$refactorable <- "Non-refactorable"
  table$refactorable[table$X.PreconditionViolations == 0 & table$X.NodeComparisons > 0 & table$X.MappedStatements > 0] <- "Refactorable"
  
  NR = table$average[table$refactorable == "Non-refactorable"]
  NRMed = median(NR)
  NRMean = mean(NR)
  maxNR = max(table$average[table$refactorable == "Non-refactorable"])
  maxR = max(table$average[table$refactorable == "Refactorable"])
  R = table$average[table$refactorable == "Refactorable"]
  RMed = median(R)
  RMean = mean(R)
  
  bwplot(
    table$average ~ table$refactorable, 
    panel = function(..., box.ratio) {
            
      panel.violin(..., col = "#cccccc", border="#777777", varwidth = FALSE, box.ratio = box.ratio, box.width=0.7)
      panel.bwplot(..., fill = "transparent", box.width=0.3, pch='|')
      
      panel.segments(x0 = 0, y0 = log10(NRMed), x1 = .85, y1 = log10(NRMed), lty = medianLty, col="#999999")
      panel.segments(x0 = 0, y0 = log10(NRMean), x1 = 1, y1 = log10(NRMean), lty = meanLty, col="#999999")
      
      panel.segments(x0 = 2.15, y0 = log10(RMed), x1 = 3, y1 = log10(RMed), lty = medianLty, col="#999999")
      panel.segments(x0 = 2, y0 = log10(RMean), x1 = 3, y1 = log10(RMean), lty = meanLty, col="#999999")
      
    },
    axis = function(side, ...) {
      switch(side, 
         left = { 
           at = c(0, log10(2), log10(20), log10(50), 2, log10(NRMed), log10(NRMean), log10(maxR), log10(maxNR));
           lbs = c(1, 2, 20, 50, 100, paste("Median=", NRMed, sep=""), paste("Mean=", round(NRMean, 1), sep=""), maxR, maxNR)
           panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = at, labels = lbs) 
         }, 
         right = { 
           atl = c(log10(RMed), log10(RMean), 0);
           lbs = c(paste("Median=", RMed, sep=""), paste("Mean=", round(RMean, 1), sep=""), "")
           panel.axis(side = side, outside = F, text.cex=0.7, rot=0, at = atl, labels = lbs)
         }, 
         axis.default(side = side, ...)) 
    },
    scales = list(
      y = list(log=T)
    ),
    par.settings = list(
      strip.background=list(col="#dddddd"),
      layout.heights = list(strip = 1.4),
      plot.symbol = list(pch = 1, col = "#888888"),
      box.rectangle = list(col = "#555555"),
      box.umbrella = list(col = "#555555"),
      layout.widths =
        list(left.padding = 0,
             key.ylab.padding = 0,
             ylab.axis.padding = 0,
             axis.key.padding = 0,
             right.padding = 0)
      #,layout.widths=list(right.padding=+5)
    ),
    ylab = ""
  )
}


